@@ -9,5 +9,6 @@ from . import uitars
 from . import omniparser
 from . import gta1
 from . import composed_grounded
+from . import glm45v
 
-__all__ = ["anthropic", "openai", "uitars", "omniparser", "gta1", "composed_grounded"]
+__all__ = ["anthropic", "openai", "uitars", "omniparser", "gta1", "composed_grounded", "glm45v"]
